/* 
    fileext.c, Copyright (C) 2003 Joe Laffey <software@laffeycomputer.com>
    http://www.laffeycomputer.com/software.html
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 

#define PROG_NAME "fileext"

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
#error Please use the configure script.
#endif

#include "filexxx.h"

void usage( void );

int main (int argc, char* argv[])
{
	char	*cp;
	char	*slashPtr; 	
        char    inStr[MAXPATHLEN +1];  
	int	isPipe;
        int     done;
	unsigned long	lineCount;

	if( argc > 2)
	{
		usage();
	}


	if( argc == 2 )
	{
		/* we have a command line arg */

		isPipe = 0;

		if( !strncmp(argv[1], "-h", 3) )
		{
			usage();
		}
		else if( !strncmp(argv[1], "-v", 3) || !strncmp(argv[1], "-l", 3) )
	        {
			fprintf(stderr, "%s v%s Copyright (c) 2003 Joe Laffey <software@laffeycomputer.com>\n", 
					PROG_NAME, VERSION);
			fprintf(stderr, "This binary compiled on %s at %s\n",
					__DATE__, __TIME__);
			fprintf(stderr, "Visit: http://www.laffeycomputer.com/software.html for updates.\n");
			fprintf(stderr, "%s comes with ABSOLUTELY NO WARRANTY; for details see the COPYING file\n", PROG_NAME);
			fprintf(stderr, "that accompained this distribution. This is free software, and you are welcome\n");
			fprintf(stderr, "to redistribute it under the terms of GNU PUBLIC LICENSE.\n");

			exit(EX_USAGE);
	
		}

		strncpy( inStr, argv[1], MAXPATHLEN);

		cp = strrchr(inStr, '.');	

                /* Be sure we are AFTER the last / */
                slashPtr = strrchr(inStr, PATH_SEPARATOR);

		if( cp != NULL && ( slashPtr == NULL || slashPtr < cp) )
		{
			/* Found the . */

			/* Print inStr from the . onward */
                        if((cp+1)[strlen(cp+1)-1] == '\n')
                                printf("%s", cp+1);
                        else
                                printf("%s\n", cp+1);

		}
		else
		{
			/* there was no . after the last path sep */
			 if(inStr[strlen(inStr)-1] == '\n')
				printf("%s", inStr);
			else
				printf("%s\n", inStr);

		}
		
		return 0;

	}
	else
        {
                /* read from stdin, no command line arg */

		isPipe = 1;
		lineCount = 0;
                done = 0;

                do 
		{
                        if( NULL == fgets(inStr, MAXPATHLEN, stdin) )
                        {
                                if( feof(stdin) )
                                {
                                        if(lineCount < 1)
                                                fprintf(stderr, "%s, EOF on read from stdin\n.", PROG_NAME);
                                        done = 1;
                                        exit(1);
                                }
                                else if( ferror(stdin) )
                                {
                                        fprintf(stderr, "%s, Read error on read from stdin\n.", PROG_NAME);
                                        exit(1);
                                }
                        
                        }
                        lineCount++;

			cp = strrchr(inStr, '.');

			/* Be sure we are AFTER the last / */
	                slashPtr = strrchr(inStr, PATH_SEPARATOR);
                         
        	        if( cp != NULL && ( slashPtr == NULL || slashPtr < cp) )
	                {
	                        /* Found the . */

	                        /* Print inStr from the . onward */
				if((cp+1)[strlen(cp+1)-1] == '\n')
	                                printf("%s", cp+1);
	                        else
	                                printf("%s\n", cp+1);
	                }
	                else
	                {
	                        /* there was no . */
				if(inStr[strlen(inStr)-1] == '\n')
		                        printf("%s", inStr);
				else
		                        printf("%s\n", inStr);
	                }
                	fflush(stdout);

                } while( !done);

        }

	

	return 0;

}


void usage(void)
{
	fprintf(stderr, "Usage: %s <pathname>\n", PROG_NAME);
	fprintf(stderr, "       -v for version / license.\n");
	fprintf(stderr, "       Will read from stdin with no argument.\n");
	exit(EX_USAGE);
}
