/* jsento.c */
/* Version 0.5.1 April 25, 2001                                           */
/* Small C hack (getting bigger) to send files to jEdit                   */
/* Original code by Stefan Radig <stefan@radig.com>	                      */
/*                                                                        */
/* USERPROFILE support, shortcut resolution, and command-line             */
/* support without full path names, as well as UNIX support by            */
/* Joe Laffey <joe@laffeycomputer.com>                                    */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                   */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>


#ifdef WIN32

#include <windows.h>
#include <windowsx.h>
#include <objbase.h>
#include <shlobj.h>
#include <initguid.h>
#include <direct.h>
#include <shellapi.h>
#include <stat.h>

#else

#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#define	SOCKET_ERROR	(-1)
#define	MAX_PATH	MAXPATHLEN

#define MAX_OPTIONS			128L 	/*maximum number of java options*/
#define MAX_OPTION_LEN		128		/* max string len for each java option */

extern char **environ;	/* for execvp()'s environment */
extern char *optarg;   
extern int optind;

#endif



static void verror_box(char *fmt, va_list args)
{
    char buffer[4096];

    vsprintf(buffer, fmt, args);
#ifdef WIN32
    MessageBox(NULL, buffer, "Error", MB_OK);
#else
	fprintf(stderr, "%s\n",buffer);
#endif

    exit(1);
}

static void error_box(char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);

    verror_box(fmt, args);
}


#ifdef WIN32
/* Resolve a Shortcut (ShellLink) */
/* Put the resolved path into outPath */
/* returns 1 on failure, zero on success */
/* Copies path to outPath on failure so */
/* outPath is always a useable path */
static int resolveLink(char* path, char* outPath)
{
	IShellLink *psl;
	HRESULT hres;
	WIN32_FIND_DATA wfd;
	char szgotpath[MAX_PATH];
	IPersistFile *ppf;

	hres = CoInitialize(NULL);
	if (!SUCCEEDED(hres))
		error_box("could not open the com library\n");

	hres = CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, &IID_IShellLink, (LPVOID *)&psl);
	if (SUCCEEDED(hres))
	{
		hres = psl->lpVtbl->QueryInterface(psl, &IID_IPersistFile, (LPVOID *)&ppf);
	   if (SUCCEEDED(hres))
		{
			WORD wsz[MAX_PATH];
		    MultiByteToWideChar(CP_ACP, 0, path, -1, wsz, MAX_PATH);
		    hres = ppf->lpVtbl->Load(ppf, wsz, STGM_READ);
			if (SUCCEEDED(hres))
			{
				hres = psl->lpVtbl->Resolve(psl, 0, SLR_ANY_MATCH);
				if (SUCCEEDED(hres))
				{
					strncpy(szgotpath, path, MAX_PATH);
				    hres = psl->lpVtbl->GetPath(psl, szgotpath, MAX_PATH, (WIN32_FIND_DATA *)&wfd, SLGP_SHORTPATH );
					if (!SUCCEEDED(hres))
						error_box("getpath failed!\n");

					if (!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
					{
						psl->lpVtbl->Release(psl);
						strncpy( outPath,szgotpath, MAX_PATH);
						return 0; /*got it*/
					}
				}
			}
			/*else
				error_box("ipersistfile load error\n");

				Commented out because this is what happens when the file is not a shortcut.
			*/

			ppf->lpVtbl->Release(ppf);
		}
		else
			error_box("queryinterface error\n");

		psl->lpVtbl->Release(psl);
	}
	else
		error_box("cocreateinstance error - hres = %08x\n", hres);

	strncpy( outPath, path, MAX_PATH);
	return 1;
}
#endif


static int contact_jedit(char* server_file_name, int argc, char** outAry, int report_minor_errors)
{
   
    FILE* server_file;
    char line[16384];
    
#ifdef WIN32
    unsigned short port;
#else
	unsigned int port;
#endif

    unsigned int key;
    int sd;
    struct sockaddr_in address;
    register int s;
    size_t	mySize;

    server_file = fopen(server_file_name, "rb");
    if (server_file == NULL)
    {
		if (report_minor_errors)
		    error_box("Could not open server file %s\nPlease setup the JEDIT_SERVER_FILE environment variable or have jEdit write its server file to the location above.", server_file_name);
		return -1;
    }

    if (fscanf(server_file, "%u\n", &port) != 1)
    {
		error_box("Wrong server file syntax: Did not find port");
		return -1;
    }
    
    if (fscanf(server_file, "%u\n", &key) != 1)
    {
		error_box("Wrong server file syntax: Did not find key");
		return -1; 
    }
    
    fclose(server_file);

    sd = socket(PF_INET, SOCK_STREAM, 0);
    address.sin_family = AF_INET;
    address.sin_addr.s_addr = htonl(0x7f000001);
    address.sin_port = htons(port);

    if (connect(sd, (struct sockaddr*)&address, sizeof(address)) == SOCKET_ERROR)
    {
		if (report_minor_errors)
		    error_box("Could not connect to jedit on port %u\n Is jEdit Running?", port);
		return -1;
    }

    sprintf(line, "%u\n--\n", key);

    send(sd, line, strlen(line), 0);
  
#ifdef WIN32
  	for (s = 0; s < argc-1; s++) /* argc DOES include the argv[0] program name */
#else
  	for (s = 0; s < argc; s++)	/* argc has been adjusted by getopt and does not include the argv[0] program name */
#endif
    {
    	mySize = sprintf(line, "%s\n", outAry[s]);
		send(sd, line, mySize, 0);
  
    }


#ifdef WIN32
    closesocket(sd);
#else
	shutdown(sd, 2);
#endif

    return 0;
}




/* This will resolve shortcuts in all the paths and check them to see if they are */
/* full paths, or relative. I will set up relative paths as full paths. */
static char** SetupPaths(int argc, char** argv, char* myDir)
{
	register int s;
	char** outAry;

	int	ch1;
	
#ifdef WIN32
    char inPath[MAX_PATH];
	char outPath[MAX_PATH];
	int ch2;
#endif
   

	
	
	outAry = malloc(sizeof(char*) * argc);
	if(outAry == NULL)
    	 	error_box("Unable to allocate memory!");

	
#ifdef WIN32
	for (s = 1; s < argc; s++)
#else
	for (s = 0; s < argc; s++)	/* already processes by getopt */
#endif
    {
    	/* Check for full path */
    	ch1 = *(argv[s]);

#ifdef WIN32
    	ch2 = *(argv[s]+1);
    	
    	if( 	( ( (ch1 >= 'a' && ch1 <='z') || (ch1 >='A' && ch1 <='Z') ) && ch2 == ':') || /* we have [a-z][A-Z]: */
    			(ch1 == '\\' && ch2 == '\\') ) /* we have a share like \\myshare */
    			
    	{
    		/* It's a full path! */	
    		resolveLink(argv[s], outPath);

    	}
    	else
    	{
    		
    		/* Not a full path so add directory */
    		strncpy(inPath, myDir, strlen(myDir)+1);
    		
    		strcat(inPath,"\\");
    		strncat(inPath, argv[s], MAX_PATH);
    		resolveLink(inPath, outPath);

    	}


		outAry[s-1] = malloc(strlen(outPath)+1);
		if(outAry[s-1] == NULL)
    	 	error_box("Unable to allocate memory!");
		
		strncpy(outAry[s-1] , outPath, strlen(outPath)+1);
		
		
#else

		outAry[s] = malloc(MAX_PATH);
		if(outAry[s] == NULL)
    	 	error_box("Unable to allocate memory!");
		
		if(realpath(argv[s], outAry[s]) == NULL)
			error_box("Could not get realpath of %s.\nError: %s\n\n\n", argv[s], strerror(errno));
		
		
#endif

    }
    return outAry;
	
}



#ifdef WIN32
/* Generate an argument list to pass to jEdit when jEdit is */
/* Not already running. These are quoted on windoze and separate by spaces. */
static char* MakeArgList(int argc, char** argv)
{
	register int s;
	char* theList;
	
#ifdef WIN32
	char outPath[MAX_PATH];
#endif
	
	
	theList = malloc(argc * (MAX_PATH + 2));  /* The +2 is for a space, and a null. */
											/* This is somewhat inefficient, should we count lengths? */
											/* Pascal strings would come in handy ;-) */
	if(theList == NULL)
		error_box("Unable to allocate memory!");
	
	theList[0]='\0'; /*set up theList as blank*/
	
#ifdef WIN32
	s = 1;
#else
	s = 0;	/* already processes by getopt */
#endif
	for (/*above*/; s < argc; s++)
    {
    
    	
#ifdef WIN32					
    	resolveLink(argv[s], outPath);
    	
		strncat( theList, "\"",1); /* add quote */
		strncat( theList, outPath, strlen(outPath));
		strncat( theList, "\" ",2); /* add quote and space at the end */
#else
		strncat( theList, argv[s], strlen(argv[s]));
		strncat( theList, " ",1);
#endif
				
   		
    }	
   
    return theList;

}
#endif

#ifndef WIN32
/* Crude splitting on spaces to parse the options enrionment variable */
static char** SpaceSplit(char* jedit_java_options,  int* optionsCount)
{
	register int i;
	register int j = 0;
	register int k = 0;
	int	theLen;	
	char** optionsAry;	

	optionsAry = malloc(MAX_OPTIONS); /*max number of options */
	
	optionsAry[j] = malloc(MAX_OPTION_LEN);
	if(optionsAry[j] ==NULL)
		error_box("Unable to allocate memory!");
	
	theLen = strlen(jedit_java_options)+1;
	for(i=0; i< theLen ; i++)
	{
		switch(jedit_java_options[i])
		{
			case ' ':	/* space */
				optionsAry[j++][k]='\0';
				k = 0;
				optionsAry[j] = malloc(MAX_OPTION_LEN);
				if(optionsAry[j] ==NULL)
					error_box("Unable to allocate memory!");
				break;
			
			case '\0': /* null */
				optionsAry[j][k]='\0';
				*optionsCount = j+1;
				break;
				
			default:
				optionsAry[j][k++] = jedit_java_options[i];
				break;
		
		}
	
	}

	return optionsAry;

}
#endif

/* This function sets up the executable string like this: */
/* $JEDIT_JAVA_EXECUTABLE $JEDIT_JAVA_OPTIONS -classpath $JEDIT_JAR_PATH:$CLASSPATH org.gjt.sp.jedit.jEdit .... */ 
#ifdef WIN32
static char* MakeExecutableString(char* argv_0)
#else
/* Unix version returns a pointer to an array of char* */
static char** MakeExecutableString(char* argv_0, int argc, char** argv)
#endif
{
	char* jedit_java_executable;
	char* jedit_java_options;
	char* jedit_jar_path;
	char* jedit_classname;
	char* classpath;
	int fatal = 0;
	int jedit_classname_was_guessed = 0;
	size_t mySize;
	char errStr[16*1024];
	
#ifdef WIN32
	char* outString;
#else
	static char** argvAry;
	char** optionsAry = NULL; /*keep compiler happy*/
	int	optionsCount;
	int offset = 0;
	int s;
#endif
	
	jedit_java_executable = getenv("JEDIT_JAVA_EXECUTABLE");

#ifdef WIN32
	/* on Windoze require this */
	if(jedit_java_executable == NULL)
		fatal = 1;
#else
	/* on unix try using "java" */
	if(jedit_java_executable == NULL)
	{
		jedit_java_executable = malloc(strlen("java")+1);
		if(jedit_java_executable == NULL)
				error_box("Unable to allocate memory!");
				
		strcpy(jedit_java_executable, "java");
	}
#endif

	jedit_java_options = getenv("JEDIT_JAVA_OPTIONS");
	
	jedit_jar_path = getenv("JEDIT_JAR_PATH");

	classpath = getenv("CLASSPATH");
	
	/* we need one of these */
	if(jedit_jar_path == NULL && classpath == NULL)
		fatal = 1;
		
	jedit_classname = getenv("JEDIT_CLASSNAME");
	if(jedit_classname == NULL)
	{
		jedit_classname_was_guessed = 1;
		jedit_classname = malloc(strlen("org.gjt.sp.jedit.jEdit")+1);
		if(jedit_classname == NULL)
				error_box("Unable to allocate memory!");
				
		strcpy(jedit_classname, "org.gjt.sp.jedit.jEdit");
	}
		
	if(fatal)
	{

		errStr[0] = '\0';
		
		strcat(errStr, "I was unable to find a running jEdit instance.\n");	
		strcat(errStr, "Additionally, I could not launch jEdit because I need some more information.\n");	
		strcat(errStr, "Please be sure the following environment variables are defined correctly:\n\n");	
		
		strcat(errStr, "\tJEDIT_JAVA_EXECUTABLE - The path to the java program itself\n");	
		if(jedit_java_executable == NULL)
			strcat(errStr, "\t  Current Value       - <undefined>\n\n");
		else
		{
			if(!strncmp(jedit_java_executable, "java", strlen("java")+1) )
				strcat(errStr, "\t  Current Value       - java (GUESSED)\n\n");	
			else
			{
				strcat(errStr, "\t  Current Value       - ");
				strcat(errStr,jedit_java_executable);	
				strcat(errStr, "\n\n");
			}
		}
			
		strcat(errStr, "\tJEDIT_JAVA_OPTIONS    - Any arguments to pass to java (optional)\n");	
		if(jedit_java_options != NULL)
		{
			strcat(errStr, "\t  Current Value       - ");
			strcat(errStr, jedit_java_options);	
			strcat(errStr,  "\n\n");
		}
		else
			strcat(errStr, "\t  Current Value       - <undefined>\n\n");
			
		strcat(errStr, "\tJEDIT_JAR_PATH        - The path to the jEdit jar file\n");	
		if(jedit_jar_path != NULL)
		{
			strcat(errStr, "\t  Current Value       - ");
			strcat(errStr, jedit_jar_path);	
			strcat(errStr,  "\n\n");
		}
		else
			strcat(errStr, "\t  Current Value       - <undefined>\n\n");
			
		strcat(errStr, "\tJEDIT_CLASSNAME        - The jEdit class name\n");	
		if(jedit_classname_was_guessed)
		{
			strcat(errStr, "\t  Current Value       - ");
			strcat(errStr, jedit_classname);
			strcat(errStr, " (GUESSED)\n\n");
		}	
		else
		{
			strcat(errStr, "\t  Current Value       - ");
			strcat(errStr, jedit_classname);	
			strcat(errStr,  "\n\n");
		}	
		
		strcat(errStr, "\tCLASSPATH             - The standard java classpath\n");	
		if(classpath != NULL)
		{
			strcat(errStr, "\t  Current Value       - ");
			strcat(errStr, classpath);	
			strcat(errStr,  "\n\n");
		}
		else
			strcat(errStr, "\t  Current Value       - <undefined>\n\n");
			
			
#ifdef WIN32
		error_box("%s", errStr);
#else
		fprintf(stderr, "%s", errStr);
		exit(1);
#endif
			
	}


#ifdef WIN32
	/* We are using a string for Windoze and an array for unix */
	mySize = 0;
	
	if(jedit_java_options!=NULL)
		mySize += strlen(jedit_java_options);
		
	if(jedit_jar_path!=NULL)
		mySize += strlen(jedit_jar_path);
		
	if(classpath!=NULL)
		mySize += strlen(classpath);
		
	
	mySize += strlen(jedit_classname);
	
	mySize += strlen("-classpath") + 128;  /* a little padding for escaping ,etc. */
	
	outString = malloc(mySize);
	if(outString == NULL)
		error_box("Unable to allocate memory!");
				
	outString[0] = '\0'; /* start out blank  - for catting  ...Meow... */

#else
	/* Unix uses an array */
	
	mySize = 	1 + 	/* argv[0] the binary's name */
				1 +	 	/* -classpath */
				1 +		/* the classpath or jedit jar as a classpath (we know we have one of these and they are put together) */
				1 +		/* the jedit classname */
				1 +		/* The terminating null pointer */	
				argc -1;	/* our args */
	
	if(jedit_java_options!=NULL)
	{
		optionsAry = SpaceSplit( jedit_java_options,  &optionsCount); /*split the options on the space character and count them */
	
		mySize += optionsCount; /* options */
	
	
	}	
	
	
	argvAry = malloc(sizeof(char*) * mySize);
	if(argvAry == NULL)
		error_box("Unable to allocate memory!");
				
	/* Now we have an array of the right number of elements */

	/* Put the executable in the zeroth element (argv[0]) for passing to execv */
	argvAry[offset] = malloc( strlen(jedit_java_executable)+1);
	if(argvAry[offset] == NULL)
		error_box("Unable to allocate memory!");
		
	strncpy(argvAry[offset++],jedit_java_executable, strlen(jedit_java_executable)+1);
#endif


	/* copy our executable path - This goes back to caller */
	strncpy(argv_0, jedit_java_executable, strlen(jedit_java_executable)+1);


	if(jedit_java_options != NULL)
	{
		
#ifdef WIN32
		/* strncat( outString,"\"", 1); */
		strncat( outString, jedit_java_options, MAX_PATH);
		/* strncat( outString,"\"", 1); */
#else
/*unix*/
		for(/* inited above */; offset <= optionsCount; offset++)
		{
			argvAry[offset] = optionsAry[offset-1];
		}
		
#endif
	
	}
		
#ifdef WIN32
	strncat( outString, " -classpath ", strlen(" -classpath "));
#else
	argvAry[offset] = malloc( strlen("-classpath")+1);
	if(argvAry[offset] == NULL)
		error_box("Unable to allocate memory!");
	
	strncpy(argvAry[offset], "-classpath", strlen("-classpath")+1);

	mySize = 1;
	if(jedit_jar_path!=NULL)
		mySize += strlen(jedit_jar_path);
		
	if(classpath !=NULL)
		mySize += strlen(classpath) + 1; /* extra 1 for the path sep */
		
	argvAry[++offset] = malloc( mySize); /* the extra +1 is for the path sep */

#endif
	
	
	if(jedit_jar_path != NULL)
	{
	
#ifdef WIN32
		strncat( outString,"\"", 1);
		strncat( outString, jedit_jar_path, MAX_PATH);	
#else
		if(argvAry[offset] == NULL)
			error_box("Unable to allocate memory!");
		
		strncpy(argvAry[offset],jedit_jar_path, strlen(jedit_jar_path)+1);
#endif
		

	}
	
	
	if(classpath != NULL)
	{
		if(jedit_jar_path != NULL)
		{
#ifdef WIN32
		strncat( outString, ";", 1);	/* win path sep */
#else
		strncat( argvAry[offset], ":", 1);	/* unix path sep */
#endif
		}

#ifdef WIN32
		strncat( outString, classpath, strlen(classpath));
#else
		strncat( argvAry[offset], classpath, strlen(classpath));
#endif
	
		
	}
		
#ifdef WIN32
		strncat( outString,"\" ", 2);

		strncat( outString, jedit_classname, strlen(jedit_classname));
#else
		argvAry[++offset] = malloc( strlen(jedit_classname)+1); 
		if(argvAry[offset] == NULL)
			error_box("Unable to allocate memory!");
		
		strncpy(argvAry[offset],jedit_classname, strlen(jedit_classname)+1);
		
		/* add our args */
#ifdef WIN32
	s = 1;
#else
	s = 0;	/* already processes by getopt */
#endif
		for (/*above*/; s < argc; s++)
    	{
 			argvAry[++offset] = argv[s];			
   		
    	}	

#endif
		
	
	


#ifdef WIN32
	/* return the string */
	return outString;
#else
	/* return the array */

	argvAry[++offset] = (char*)0L; /* terminate the arg list with a null pointer */
	return argvAry;
#endif
}



#ifdef WIN32
/* Just return a string value for an error code */
static char* shell_execute_result_to_string(int result)
{
    switch (result)
    {
	case 0: return "The operating system is out of memory or resources.";
	case ERROR_FILE_NOT_FOUND: return "The specified file was not found.";
	case ERROR_PATH_NOT_FOUND: return "The specified path was not found.";
	case ERROR_BAD_FORMAT: return "The .exe file is invalid (non-Win32 .exe or error in .exe image).";
	case SE_ERR_ACCESSDENIED: return "The operating system denied access to the specified file.";
	case SE_ERR_ASSOCINCOMPLETE: return "The file name association is incomplete or invalid.";
	case SE_ERR_DDEBUSY: return "The DDE transaction could not be completed because other DDE transactions were being processed.";
	case SE_ERR_DDEFAIL: return "The DDE transaction failed.";
	case SE_ERR_DDETIMEOUT: return "The DDE transaction could not be completed because the request timed out.";
	case SE_ERR_DLLNOTFOUND: return "The specified dynamic-link library was not found.";
	case SE_ERR_NOASSOC: return "There is no application associated with the given file name extension.";
	case SE_ERR_OOM: return "There was not enough memory to complete the operation.";
	case SE_ERR_SHARE: return "A sharing violation occurred.";
	default: return "Unexpected error!!!";
    }
}
#endif


/* list how we are used */
static void usage(char* myName)
{
#ifdef WIN32
	error_box("Usage: %s <file1> <file2> ...", myName);
#else
	error_box("Usage: %s -[rq] <file1> <file2> ...", myName);
#endif
}






/* Main Entry */
#ifdef WIN32
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nShowCmd)
#else
int main(int argc, char** argv)
#endif
{


#ifdef WIN32
    extern int __argc;
    extern char **__argv;
    WORD version;
    WSADATA wsa_data;
    int argc = __argc;
    char** argv = __argv;
    int result;
    char* argList;
    char* execList;
    char* fullList;
#else
	char** argvAry;
	int doRootCheck = 1;
	int quiet = 0;
	char line[256];
	int ch;
#endif

	struct stat	fileStat;
	register int i;
	static char jedit_java_executable[MAX_PATH]; 

    char* server_file_name;
    char* server_file_name_tmp;
   
    size_t	mySize;
	
    char** outAry;
	char myDir[MAX_PATH];

#ifdef WIN32
    version = MAKEWORD(1, 0);
#else
	/* unix - so get options */
	 while ((ch = getopt(argc, argv, "rq")) != -1)
	 {		   
		/* get our options */
		switch(ch) 
		{
			case 'r':
							doRootCheck = 0;
							break;
			case 'q':
							quiet = 1;
							break;
							
			default:		usage(argv[0]);
		}
	}
	argc -= optind;
	argv += optind;
#endif
	
	
	
#ifdef WIN32
	if (argc < 2)
		usage(argv[0]);
#else
	if (argc <1)
		usage((char*)*(argv - optind));	
#endif

	
	/* check for files */	

#ifdef WIN32
	i=1;
#else
	i=0;	/* getopt removed argv[0] - our program name */
#endif
	for(/*above*/; i < argc ; i++)
	{
			if(stat(argv[i], &fileStat))	/* be sure files exist */
			{				
#ifdef WIN32		
					error_box("\n%s: File \"%s\" not found.\n", argv[0], argv[i] );
#else
					error_box("\n%s: File \"%s\" not found.\n", (char*)*(argv-optind), argv[i] );
#endif
					exit(-1);
			}
			else
			{			
				if((fileStat.st_mode) & S_IFDIR)
				{
#ifdef WIN32
					error_box("\n%s: I cannot open the directory \"%s\".\n", argv[0], argv[i] );
#else
					error_box("\n%s: I cannot open the directory \"%s\".\n", (char*)*(argv-optind), argv[i] );
#endif
					exit(-1);
				}
			}
	}
	

	
	
	#ifdef WIN32
	if (WSAStartup(version, &wsa_data) != 0)
		error_box("Could not start winsock");
#endif


    server_file_name = getenv("JEDIT_SERVER_FILE");
    if (server_file_name == NULL)
    {
		
#ifdef WIN32
    	 server_file_name_tmp =  getenv("USERPROFILE");
#else 
		 server_file_name_tmp =  getenv("HOME");
#endif

    	 if (server_file_name_tmp == NULL)
    	 	error_box("Did not find environment variable JEDIT_SERVER_FILE, and could not find USERPROFILE!");

    	 mySize = strlen(server_file_name_tmp)+15L;
    	 server_file_name = malloc(mySize);
    	 if (server_file_name == NULL)
    	 	error_box("Could not allocate memory!");

    	 strncpy(server_file_name, server_file_name_tmp, mySize);
#ifdef WIN32
    	 strncat(server_file_name, "\\.jedit\\server", 15L); /* the length - ignore one of each of the doulbe backslashes */
#else
    	 strncat(server_file_name, "/.jedit/server", 15L); /* the length */
#endif

    }

	getcwd(myDir, MAX_PATH);

	outAry = SetupPaths(argc, argv, myDir);

    if (contact_jedit(server_file_name, argc, outAry, 0) < 0)
    {
		/*Could not contact jEdit via the socket. We will try to launch it. */
		
		
		
		
#ifdef WIN32 


		execList = MakeExecutableString((char*)&jedit_java_executable);
		
		argList = MakeArgList(argc, argv);
		
		fullList = malloc(strlen(execList) + strlen(argList) +1);
		if(fullList ==NULL)
			error_box("Unabale to allocate memory!");
			
		strncpy(fullList, execList, strlen(execList)+1);
		strncat(fullList, " ", 1);
		strncat(fullList, argList, strlen(argList));
	
		/* Execute Launch under Win 32 */   	 	
		result = (int)ShellExecute(NULL, "open", jedit_java_executable, fullList, myDir, SW_SHOWNORMAL);
		if (result < 32)
		{
		    error_box("Could not start jEdit server (%s): %s", jedit_java_executable, shell_execute_result_to_string(result));
		}
		
#else

		/* Execute launch under unix */

		/* Should check for root I think... */
		if(doRootCheck && getuid()==0)
		{
			fprintf(stderr, "Really run jEdit as ROOT? (Use -r to avoid this warning.) [y/N] ");
							
			fgets(line, 255, stdin);		/*read in response */
									
							
			if( line[0] != 'y' && line[0] != 'Y' )
			{
					fprintf(stderr,"\n%s:	User cancelled operation.\n", (char*)*(argv-optind));
					exit(-1);
			}
		}
		
		/* Setup the executable string like this: */
		/* $JEDIT_JAVA_EXECUTABLE $JEDIT_JAVA_OPTIONS -classpath $JEDIT_FILE:$CLASSPATH org.gjt.sp.jedit.jEdit .... */ 
		
		argvAry = MakeExecutableString(jedit_java_executable, argc, argv);
		
		if(!fork())
		{
			/* child */
			if(execvp(jedit_java_executable, argvAry) == -1)
			{
				error_box("An Error Occurred while trying to execute %s\nError Was %s", jedit_java_executable, strerror(errno) );
			}
		}
	
		if(!quiet)
			printf("\nI could not find a running jEdit instance...\nSo I took the liberty of launching one!\n");
	
#endif
		
		/* We have succesfully launched jEdit, passing the file paths as parameters (argv style) */
		exit(0);
		
    }
    

	/* We succeeded from above, passing the file paths using the socket. */
    return(0);
}





