/* jsento.c */
/* April 23, 2001                                                         */
/* Small C hack (getting bigger) to send files to jEdit                   */
/* Original code by Stefan Radig <stefan@radig.com>	                      */
/*                                                                        */
/* USERPROFILE support, shortcut resolution, and command-line             */
/* support without full path names, as well as UNIX support by            */
/* Joe Laffey <joe@laffeycomputer.com>                                    */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                   */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>



#ifdef WIN32

#include <windows.h>
#include <windowsx.h>
#include <objbase.h>
#include <shlobj.h>
#include <initguid.h>
#include <direct.h>

#else

#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#define	SOCKET_ERROR	(-1)
#define	MAX_PATH	MAXPATHLEN

#endif



static void verror_box(char *fmt, va_list args)
{
    char buffer[4096];

    vsprintf(buffer, fmt, args);
#ifdef WIN32
    MessageBox(NULL, buffer, "Error", MB_OK);
#else
	fprintf(stderr, "%s\n",buffer);
#endif

    exit(1);
}

static void error_box(char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);

    verror_box(fmt, args);
}


#ifdef WIN32
/* Resolve a Shortcut (ShellLink) */
/* Put the resolved path into outPath */
/* returns 1 on failure, zero on success */
/* Copies path to outPath on failure so */
/* outPath is always a useable path */
static int resolveLink(char* path, char* outPath)
{
	IShellLink *psl;
	HRESULT hres;
	WIN32_FIND_DATA wfd;
	char szgotpath[MAX_PATH];
	IPersistFile *ppf;

	hres = CoInitialize(NULL);
	if (!SUCCEEDED(hres))
		error_box("could not open the com library\n");

	hres = CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, &IID_IShellLink, (LPVOID *)&psl);
	if (SUCCEEDED(hres))
	{
		hres = psl->lpVtbl->QueryInterface(psl, &IID_IPersistFile, (LPVOID *)&ppf);
	   if (SUCCEEDED(hres))
		{
			WORD wsz[MAX_PATH];
		    MultiByteToWideChar(CP_ACP, 0, path, -1, wsz, MAX_PATH);
		    hres = ppf->lpVtbl->Load(ppf, wsz, STGM_READ);
			if (SUCCEEDED(hres))
			{
				hres = psl->lpVtbl->Resolve(psl, 0, SLR_ANY_MATCH);
				if (SUCCEEDED(hres))
				{
					strncpy(szgotpath, path, MAX_PATH);
				    hres = psl->lpVtbl->GetPath(psl, szgotpath, MAX_PATH, (WIN32_FIND_DATA *)&wfd, SLGP_SHORTPATH );
					if (!SUCCEEDED(hres))
						error_box("getpath failed!\n");

					if (!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
					{
						psl->lpVtbl->Release(psl);
						strncpy( outPath,szgotpath, MAX_PATH);
						return 0; /*got it*/
					}
				}
			}
			/*else
				error_box("ipersistfile load error\n");

				Commented out because this is what happens when the file is not a shortcut.
			*/

			ppf->lpVtbl->Release(ppf);
		}
		else
			error_box("queryinterface error\n");

		psl->lpVtbl->Release(psl);
	}
	else
		error_box("cocreateinstance error - hres = %08x\n", hres);

	strncpy( outPath, path, MAX_PATH);
	return 1;
}
#endif


static int contact_jedit(char* server_file_name, int argc, char** outAry, int report_minor_errors)
{
   
    FILE* server_file;
    char line[16384];
    
#ifdef WIN32
    unsigned short port;
#else
	unsigned int port;
#endif

    unsigned int key;
    int sd;
    struct sockaddr_in address;
    register int s;
    size_t	mySize;

    server_file = fopen(server_file_name, "rb");
    if (server_file == NULL)
    {
		if (report_minor_errors)
		    error_box("Could not open server file %s\nPlease setup the JEDIT_SERVER_FILE environment variable or have jEdit write its server file to the location above.", server_file_name);
		return -1;
    }

    if (fscanf(server_file, "%u\n", &port) != 1)
    {
		error_box("Wrong server file syntax: Did not find port");
		return -1;
    }
    
    if (fscanf(server_file, "%u\n", &key) != 1)
    {
		error_box("Wrong server file syntax: Did not find key");
		return -1;
    }
    
    fclose(server_file);

    sd = socket(PF_INET, SOCK_STREAM, 0);
    address.sin_family = AF_INET;
    address.sin_addr.s_addr = htonl(0x7f000001);
    address.sin_port = htons(port);

    if (connect(sd, (struct sockaddr*)&address, sizeof(address)) == SOCKET_ERROR)
    {
		if (report_minor_errors)
		    error_box("Could not connect to jedit on port %u\n Is jEdit Running?", port);
		return -1;
    }

    sprintf(line, "%u\n--\n", key);

    send(sd, line, strlen(line), 0);
  
  
  	for (s = 0; s < argc-1; s++)
    {
    	mySize = sprintf(line, "%s\n", outAry[s]);
		send(sd, line, mySize, 0);
  
    }


#ifdef WIN32
    closesocket(sd);
#else
	shutdown(sd, 2);
#endif

    return 0;
}


/* This will resolve shortcuts in all the paths and check them to see if they are */
/* full paths, or relative. I will set up relative paths as full paths. */
static char** SetupPaths(int argc, char** argv, char* myDir)
{
	register int s;
	char** outAry;
    char inPath[MAX_PATH];
	
#ifdef WIN32
	char outPath[MAX_PATH];
#endif
   
	int	ch1;
	int ch2;
	
	outAry = malloc(argc);
	if(outAry == NULL)
    	 	error_box("Unable to allocate memory!");

	

 	for (s = 1; s < argc; s++)
    {
    	/* Check for full path */
    	ch1 = *(argv[s]);

#ifdef WIN32
    	ch2 = *(argv[s]+1);
    	
    	if( 	( ( (ch1 >= 'a' && ch1 <='z') || (ch1 >='A' && ch1 <='Z') ) && ch2 == ':') || /* we have [a-z][A-Z]: */
    			(ch1 == '\\' && ch2 == '\\') ) /* we have a share like \\myshare */
    			
#else
		if( ch1 == '/' )
#endif
    	{
    	
    		/* It's a full path! */
    		
#ifdef WIN32
    		resolveLink(argv[s], outPath);
#else
			strncpy(inPath, argv[s], strlen(argv[s])+1);
#endif

    	}
    	else
    	{
    		
    		/* Not a full path so add directory */
    		strncpy(inPath, myDir, strlen(myDir)+1);
    		
#ifdef WIN32
    		strcat(inPath,"\\");
#else
			strcat(inPath,"/");
#endif

    		strncat(inPath, argv[s], MAX_PATH);
#ifdef WIN32
    		resolveLink(inPath, outPath);
#endif

    	}

		
#ifdef WIN32

		outAry[s-1] = malloc(strlen(outPath)+1);
		if(outAry[s-1] == NULL)
    	 	error_box("Unable to allocate memory!");
		
		strncpy(outAry[s-1] , outPath, strlen(outPath)+1);
		
#else

		outAry[s-1] = malloc(strlen(inPath)+1);
		if(outAry[s-1] == NULL)
    	 	error_box("Unable to allocate memory!");
		
		strncpy(outAry[s-1] , inPath, strlen(inPath)+1);
		
#endif

    }
    return outAry;
	
}



/* Generate an argument list to pass to jEdit when jEdit is */
/* Not already running. These are quoted and separate by spaces. */
static char* MakeArgList(int argc, char** argv)
{
	register int s;
	char* theList;
	
#ifdef WIN32
	char outPath[MAX_PATH];
#endif
	
	
	theList = malloc(argc * (MAX_PATH+2));  /* The +2 is for a space, and a null. */
											/* This is somewhat inefficient, should we count lengths? */
											/* Pascal strings would come in handy ;-) */
	if(theList == NULL)
		error_box("Unable to allocate memory!");
	
	theList[0]='\0'; /*set up theList as blank*/
	
	for (s = 1; s < argc; s++)
    {
    
#ifdef WIN32					
    	resolveLink(argv[s], outPath);
		strncat( theList, outPath, strlen(outPath));
#else
		strncat( theList, argv[s], strlen(argv[s]));
#endif

    		
    								
   		strncat( theList, " ",2); /* add space at the end */
    }	
   
    return theList;

}



#ifdef WIN32
/* Just return a string value for an error code */
static char* shell_execute_result_to_string(int result)
{
    switch (result)
    {
	case 0: return "The operating system is out of memory or resources.";
	case ERROR_FILE_NOT_FOUND: return "The specified file was not found.";
	case ERROR_PATH_NOT_FOUND: return "The specified path was not found.";
	case ERROR_BAD_FORMAT: return "The .exe file is invalid (non-Win32 .exe or error in .exe image).";
	case SE_ERR_ACCESSDENIED: return "The operating system denied access to the specified file.";
	case SE_ERR_ASSOCINCOMPLETE: return "The file name association is incomplete or invalid.";
	case SE_ERR_DDEBUSY: return "The DDE transaction could not be completed because other DDE transactions were being processed.";
	case SE_ERR_DDEFAIL: return "The DDE transaction failed.";
	case SE_ERR_DDETIMEOUT: return "The DDE transaction could not be completed because the request timed out.";
	case SE_ERR_DLLNOTFOUND: return "The specified dynamic-link library was not found.";
	case SE_ERR_NOASSOC: return "There is no application associated with the given file name extension.";
	case SE_ERR_OOM: return "There was not enough memory to complete the operation.";
	case SE_ERR_SHARE: return "A sharing violation occurred.";
	default: return "Unexpected error!!!";
    }
}
#endif









/* Main Entry */
#ifdef WIN32
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nShowCmd)
#else
int main(int argc, char** argv)
#endif
{


#ifdef WIN32
    extern int __argc;
    extern char **__argv;
    WORD version;
    WSADATA wsa_data;
    int argc = __argc;
    char** argv = __argv;
    int result;
#endif


    char* server_file_name;
    char* server_file_name_tmp;
   
    size_t	mySize;
    char* jedit_executable_path;
    char** outAry;
	char myDir[MAX_PATH];

#ifdef WIN32
    version = MAKEWORD(1, 0);
#endif

    if (argc < 2)
		error_box("Usage: %s <file1> <file2> ...", argv[0]);

#ifdef WIN32
    if (WSAStartup(version, &wsa_data) != 0)
		error_box("Could not start winsock");
#endif


    server_file_name = getenv("JEDIT_SERVER_FILE");
    if (server_file_name == NULL)
    {
		
#ifdef WIN32
    	 server_file_name_tmp =  getenv("USERPROFILE");
#else 
		 server_file_name_tmp =  getenv("HOME");
#endif

    	 if (server_file_name_tmp == NULL)
    	 	error_box("Did not find environment variable JEDIT_SERVER_FILE, and could not find USERPROFILE!");

    	 mySize = strlen(server_file_name_tmp)+15L;
    	 server_file_name = malloc(mySize);
    	 if (server_file_name == NULL)
    	 	error_box("Could not allocate memory!");

    	 strncpy(server_file_name, server_file_name_tmp, mySize);
#ifdef WIN32
    	 strncat(server_file_name, "\\.jedit\\server", 15L); /* the length - ignore one of each of the doulbe backslashes */
#else
    	 strncat(server_file_name, "/.jedit/server", 15L); /* the length - ignore one of each of the doulbe backslashes */
#endif

    }

	getcwd(myDir, MAX_PATH);

	outAry = SetupPaths(argc, argv, myDir);

    if (contact_jedit(server_file_name, argc, outAry, 0) < 0)
    {
		jedit_executable_path = getenv("JEDIT_EXECUTABLE_PATH");
    	if (jedit_executable_path == NULL)
    	 	error_box("Did not find environment variable JEDIT_EXECUTABLE_PATH to start jEdit!");
    	 	
#ifdef WIN32 
		/* Execute Launch under Win 32 */   	 	
		result = (int)ShellExecute(NULL, "open", jedit_executable_path, MakeArgList(argc, argv), myDir, SW_SHOWNORMAL);
		if (result < 32)
		{
		    error_box("Could not start jEdit server (%s): %s", jedit_executable_path, shell_execute_result_to_string(result));
		    exit(1);
		}
#else

		/* Execute launch under unix */

		/* Should check for root I think... */
		
		
		if(!fork())
		{
			/* child */
			if(execl(jedit_executable_path, jedit_executable_path, MakeArgList(argc, argv), (char*)0L ) == -1)
			{
				error_box("An Error Occurred while trying to execute %s\nError Was %s", jedit_executable_path, strerror(errno) );
				exit(1);
			}
		}
	
		error_box("I could not find a running jEdit instance...\nSo I took the liberty of launching one!");
	
#endif
		
		/* We have succesfully launched jEdit, passing the file paths as parameters (argv style) */
		exit(0);
		
    }
    
    if (contact_jedit(server_file_name, argc, outAry, 1) < 0)
		exit(1);

    exit(0);
}



