#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/uio.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>
#include <sysexits.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>


/* This program makes a little character spinner. */
/* e.g. - \ | / - \ | / ...                       */
/* It is useful for keeping links alive, etc.     */
                          


/*
spinner.c
vers 1.
$Revision: 1.2 $

    Copyright (C) 2002 Joe Laffey <joe@-REMOVE-THIS-laffeycomputer.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef VERSION
#define VERSION "1.1"
#endif

#ifndef PACKAGE
#define PACKAGE "spinner"
#endif

/* deafult delay in seconds */
#define	DEFAULT_DELAY		2

/*default process priority */
#define DEFAULT_PRIORITY	10

/* the name of the env var to set to our pid */
/* Not a "." is added to the front of this below */
#define	DEFAULT_PID_FILENAME	"spinner.pid"

/* path separator character - ought to determine dynamically */
#define PATH_SEPARATOR		'/'

/* max length of the fail message */
#define MAX_FAIL_MSG_LEN	256

/* maximum length of the TERM env var - bounds are checked */
#define MAX_TERM_TYPE_LEN	64

#define MAXINT				(~((unsigned int)1 << ((8 * sizeof(int)) - 1)))
#define ESCAPE	 			27


/* max len of output string.. change if strings are changed below ( for cache )*/
#define	OUTPUT_STR_MAX	16


/* Function prototypes */
static void usage(void);
static void DoInterrupt(void);
static int Daemonize(int, int, char*);
static void License(void);

/****************/
/* Static vars  */

/* the file descriptor of the terminal to which we write */
static int ttyDes;
static int interrupted = 0;




/***************************/
/* Print usage info        */
static void usage(void)
{
	
	fprintf(stderr, "------------------------------------------------\n");
	fprintf(stderr, PACKAGE " " VERSION  " by Joe Laffey, LAFFEY Computer Imaging.\n");
	fprintf(stderr, "Visit http://www.laffeycomputer.com/ for updates.\n");
	fprintf(stderr, "This software comes with ABSOLUTELY NO WARRANTY. " PACKAGE " -L for license.\n");   
    fprintf(stderr, "Usage:  " PACKAGE "[-IutTv[p|P][f|F]] [delay]\n");
	fprintf(stderr, "        -I   Do *not* use inverse video for spinner\n");
	fprintf(stderr, "        -u   Delay is in microseconds instead of seconds\n");	
	fprintf(stderr, "        -T   Ignore incompatible TERM environment var setting\n");
	fprintf(stderr, "        -v   Verbose mode (lots of output)\n");
	fprintf(stderr, "        -P   Do *not* change process priority (default is to make nice)\n");
	fprintf(stderr, "        -p   <priority> Specify process priority to use.\n");
	fprintf(stderr, "        -f   <path> Set pid file path (default is $HOME/." DEFAULT_PID_FILENAME ")\n");
	fprintf(stderr, "        -F   Do *not* create a pid file\n");
	fprintf(stderr, "        -t   <tty path> Specify path to tty to write to\n");
	fprintf(stderr, "        -r   Reset term on quit (Use if you get left in inverse a lot.)\n");
	fprintf(stderr, "        -R   Reset the term and Quit (nothing else)\n");
	fprintf(stderr, "        -L   Display the license.\n");
	fprintf(stderr, "Returns: 0 on success, non-zero on failure.\n");
	fprintf(stderr, "Launches into the background on success.\n");
	fprintf(stderr, "Use: kill `cat <pidfile>` to stop.\n");
	fprintf(stderr, "Delays shorter than 1 sec may cause display update issues!\n");
	

	exit(EX_USAGE);
}



/*****************************/
/*  Print out the license */
void License(void)
{
	fprintf(stderr, "------------------------------------------------\n");
	fprintf(stderr, PACKAGE " " VERSION " by Joe Laffey, LAFFEY Computer Imaging.\n");
	fprintf(stderr, "Visit http:// www.laffeycomputer.com/ for updates.\n");
	fprintf(stderr, "This software is copyright 2002 by Joe Laffey.\n\n");   
    
	fprintf(stderr, "%s comes with ABSOLUTELY NO WARRANTY; for details see the COPYING file\nthat accompained this distribution. ", PACKAGE);
	fprintf(stderr, "This is free software, and you are welcome\nto redistribute it");
	fprintf(stderr, " under the terms of GNU PUBLIC LICENSE.\n");


	exit(EX_USAGE);
}





/****************************/
/*  Handle control-Cs       */
static void DoInterrupt(void)
{
	interrupted++;
	if(interrupted > 2)
	{
		/* reset term if possible */
		write(ttyDes, "\0338", 2);
		write(ttyDes, "\033c", 2);
		exit(EX_TEMPFAIL);
	}

}

/****************************/
/*  Become a daemon proc       */
static int Daemonize(int verbose, int doPidFile, char* pidFileName)
{
	pid_t pid;
	
	if( (pid = fork()) < 0 )
		return (-1);
	else if( pid != 0 )
	{
		if(doPidFile)
		{
			
			FILE*	pidFile;
			
			pidFile = fopen(pidFileName, "w");
			if(pidFile == NULL)
			{
				fprintf(stderr, "%s: WARNING: Unable to open pid file %s for writing.\n%s: WARNING: No pid file will be written.\n", PACKAGE, pidFileName, PACKAGE);	
				goto error; /* don't try to write */
			}
			
			if(verbose)
				fprintf(stderr, "%s: Writing daemon's pid to pid file %s.\n", PACKAGE, pidFileName);	
				
			/* poorly assume %d is good for a pid */
			fprintf(pidFile, "%d\n", pid );
			
			fclose(pidFile);
			
			
			error:
			/* jump out of here in case of error*/
			;
		}
		
		
		/* poorly assume %d is good for a pid */
		if(verbose)
			fprintf(stderr, "%s: Parent process terminating.\n%s: Child process ( pid %d ) continuing...\n", PACKAGE, PACKAGE, pid);	
		
		exit(0); /*parent exits stage left */
	}	
	/* now child */
	setsid();	/* become sess leader */
	
	chdir("/");	/* chdr to root to allow unmounting of filesystems, etc. */
	
	umask(0);	
	
	return(0);

}





/****************************/
/* Main Entry               */
int main(int argc, char** argv)
{
	/* The spinning characters */
	char chars[] = { '-', '\\', '|','/','-','\\','|','/' };

	/* length of above */
	int numChars = sizeof(chars);
	
	/* holds the pre-built strings of code to display */
	char cache[sizeof(chars)][OUTPUT_STR_MAX + 1];

	/* length of the cached strings. They are all the same */
	int strLen = 0; /* defaulf to quell warnings*/
	
	/* count.. */
	unsigned int i = 0; /* defaulf to quell warnings*/

	/* time of sleep in secs or microseconds depending on switch*/
	int time;
	
	/*The nae of the original tty we opened.. Stored for when it dies */
	char ttyName[_POSIX_PATH_MAX + 1]; 
	
	/* incoming name of tty to open from -t option */
	char whichTTY[_POSIX_PATH_MAX + 1];
	
	/* for getopt */
	int ch;
	
	/* should we use microtime for delay ? */
	int microTime = 0;
	
	/* Should we inverse the spinner ? */
	int inverse = 1;
	
	/* should we open another TTY other than DEFAULT_TTY ?*/
	int useTTY = 0;
	
	/* print extra garbage */
	int verbose = 0;
	
	/* Should we set the priority? defaults to a lower priority */
	int setPriority = 1;
	
	/* The default value to use for priority */
	int priority = DEFAULT_PRIORITY;
	
	/* Set the envrionment var to contain the child PID ? */
	int doPidFile = 1;
	
	/* hold the file path to the pid file */
	char pidFileName[_POSIX_PATH_MAX + 1] = "";
	
	/* did the user specify a pid file ? */
	int pidFileSet = 0;
	
	/* hold a failure message for later display after reset */
	char failMsg[MAX_FAIL_MSG_LEN +1] = "";
	
	/* the exit code to return */
	int exitCode = 0;
	
	/* Should we try to reset the term on quit ? */
	int resetOnQuit = 0;
	
	/* Should we try to reset the term ONLY and then quit (do nothing else)? */
	int resetOnly = 0;
	
	/* Should we check the TERM env var to be vt100 or vt102 ? */
	int checkTermType = 1;
	
	/* the TERM string from the env */
	char* termType;
	
	/* the HOME string from the env */
	char* homePath;
	
	while ((ch = getopt(argc, argv, "Iut:vp:Pf:FRTL")) != -1)		/* get our options */
		switch(ch) {
		case 'I':
				 inverse = 0;
				
				 break;
		case 'u':
				/* delay should be in microseconds as opposed to default of seconds */
				microTime = 1;
				break;
		case 't':
				useTTY = 1;
				strncpy(whichTTY, optarg, sizeof(whichTTY));
				break;
		case 'v':
				verbose = 1;
				break;
		case 'P':
				setPriority = 0;
				break;
		case 'p':
				priority = atoi(optarg);;
				break;
		case 'F':
				doPidFile = 0;
				break;
		case 'f':
				/* pid file path */
				pidFileSet = 1;
				strncpy(pidFileName, optarg, _POSIX_PATH_MAX);
				break;
		case 'R':
				/* reset only*/
				interrupted = 1; /* hack to skip the main loop */
				resetOnQuit = 1;
				resetOnly = 1;
				break;		
		case 'r':
				resetOnQuit = 1;
				break;		
		case 'T':
				checkTermType = 0;
				break;	
				
		case 'L':
				License();
				break;	
				
		default:
				 usage();
	 }
	 argc -= optind;
	 argv += optind;

	
	if(argc > 2)
		usage();
	
	
	/* Check term type */
	if(checkTermType)
	{
		
		if(NULL == (termType = getenv( "TERM" ) ))
		{
			fprintf(stderr, "%s: Unable to determine TERM type from the environment.\n%s: Use -T switch to override check and use VT100 codes anyway.\n", PACKAGE, PACKAGE);
			exit(1);
		}
		
	
		if( ! ( (0 == strncasecmp(termType, "VT100", sizeof("VT100") )) ||  (0 == strncasecmp(termType, "VT102", sizeof("VT102") )) ) )
		{
			fprintf(stderr, "%s: Your TERM type from the environment (%s)\n%s: does not appear to be VT100 or VT102.\n%s: Use -T switch to override check and use VT100 codes anyway.\n", PACKAGE, termType, PACKAGE, PACKAGE);
			exit(1);
		}
	}
	
	if(doPidFile && !pidFileSet)
	{
		/* setup the default filename */
		if(NULL == (homePath = getenv( "HOME" ) ))
		{
			fprintf(stderr, "%s: WARNING: Unable to determine HOME directory.\n%s: WARNING: Pid file will be written to current directory as %s\n", PACKAGE, PACKAGE, DEFAULT_PID_FILENAME);
		}
		else
		{
			int len;
			
			strncpy(pidFileName, homePath , _POSIX_PATH_MAX);
			
			pidFileName[_POSIX_PATH_MAX] = '\0'; /* Terminate ( see Arnold, et. al. ) */
			
			len = strlen(pidFileName);
			
			if( (pidFileName[len] != PATH_SEPARATOR) && (len < _POSIX_PATH_MAX) )
			{
				/* add a slash */
				pidFileName[len] = PATH_SEPARATOR;
				pidFileName[len + 1] = '\0'; /* Terminate ( see Arnold, et. al. ) */
			}
			
			/* add the dot */
			/* We do this here because we don't want the emergency pid file to have a dot, lest we have these lying around */
			strncat(pidFileName, ".", _POSIX_PATH_MAX - strlen(pidFileName) ); 
		}
		
		strncat(pidFileName, DEFAULT_PID_FILENAME, (_POSIX_PATH_MAX - strlen(pidFileName) - 1) );
	}
	
	if(verbose)
	{
		fprintf(stderr, "%s: Inverse mode %s.\n", PACKAGE, (inverse? "enabled":"disabled"));
		if(useTTY)
			fprintf(stderr, "%s: Using user-selected TTY: %s.\n", PACKAGE, whichTTY);
		
		if(resetOnQuit)
			fprintf(stderr, "%s: Will attempt to reset the terminal on quit.\n", PACKAGE);
		else
			fprintf(stderr, "%s: Will not attempt to reset the terminal on quit.\n", PACKAGE);
			
		if(setPriority)
			fprintf(stderr, "%s: Will attempt to set process priority to %d.\n", PACKAGE, priority);
		else
			fprintf(stderr, "%s: Will not alter process priority.\n", PACKAGE);	
	}
	
	if(argc <  1)
	{
		time = DEFAULT_DELAY;	
		if(verbose)
			fprintf(stderr, "%s: No delay specified. Using default delay.\n", PACKAGE);
	
	}	
	else
	{
		/* nab the delay from arg */
		time = atoi(argv[0]);
	}

	if(verbose)
		fprintf(stderr, "%s: Using %d %ssec delay.\n", PACKAGE, time, (microTime?"u":""));
	
	if(!useTTY)
	{
		/* use our input tty. We cannot use /dev/tty here because when we 	*/
		/* become a daemon below it is no longer valid!!					*/
		strncpy(whichTTY, ttyname(STDIN_FILENO), _POSIX_PATH_MAX);
	}

	
	if(verbose)
		fprintf(stderr, "%s: Opening TTY: %s.\n", PACKAGE, whichTTY);
		
	ttyDes = open( whichTTY, O_WRONLY );
	if(ttyDes < 0)
	{
		fprintf(stderr, "%s: unable to open %s for writing!\n%s: %s\n", argv[0], argv[2], argv[0], strerror(errno));
		exit(1);
	}
	

	
	/* Test to see if we opened a terminal 						*/
	/* Note this test sometimes seems to fail 					*/
	/* i.e. isatty() returns true for a terminal not in use. 	*/
	if(!isatty(ttyDes))
	{
		fprintf(stderr, "%s: %s is not an active terminal!\n", argv[0], argv[2]);
		exit(1);
	}
	
		
	/* get the name of our term */
	strncpy(ttyName, ttyname(ttyDes), _POSIX_PATH_MAX);
	
	if(!resetOnly)
	{
		if(verbose)
			fprintf(stderr, "%s: Generating display strings.\n", PACKAGE);
				
		/*We pre-cache all the character strings for the spinner */
		for(i = 0; i < numChars; i++)
		{	
			/* These sprintfs create the strings that get sent to the term. */
			/* They include vt100 escape codes. 							*/
			/* ^[    = ESCAPE												*/
			/* ^[7   = store pos and settings								*/
			/* ^[[H  = home													*/
			/* ^[[7m = inverse video										*/
			/* ^[8   = restore pos and settings								*/
		
			if(inverse)
				sprintf(cache[i], "%c7%c[H%c[7m%c%c8", ESCAPE, ESCAPE, ESCAPE, chars[i], ESCAPE);
			else
				sprintf(cache[i], "%c7%c[H%c%c8", ESCAPE, ESCAPE, chars[i], ESCAPE);
		}
		
		/* length of the cacched strings.. They are all the same length */
		strLen = strlen(cache[0]); 
		
	
		/*  Set up our signal handlers... */
		if( SIG_ERR == signal(SIGINT, (void *)DoInterrupt))
		{
			fprintf(stderr, "%s: Unable to install SIGINT signal handler!\n%s: %s\n", PACKAGE, PACKAGE, strerror(errno));
			exit(1);
		}
		
		if( SIG_ERR == signal(SIGTERM, (void *)DoInterrupt))
		{
			fprintf(stderr, "%s: Unable to install SIGTERM signal handler!\n%s: %s\n", PACKAGE, PACKAGE, strerror(errno));
			exit(1);
		}
		
		/* We really ought not be receiving this, since we go daemon... */
		if( SIG_ERR == signal(SIGHUP, (void *)DoInterrupt))
		{
			fprintf(stderr, "%s: Unable to install SIGHUP signal handler!\n%s: %s\n", PACKAGE, PACKAGE, strerror(errno));
			exit(1);
		}
		
		
		if(setPriority)
		{
			if(verbose)
				fprintf(stderr, "%s: Setting process priority to %d.\n", PACKAGE, priority);
		
			if( -1 == setpriority( PRIO_PROCESS, 0 /* our process */, priority ) )
			{
				fprintf(stderr, "%s: WARNING: Unable to set process priority to %d.\n%s: WARNING: %s\n", PACKAGE, priority, PACKAGE, strerror(errno));
			}
		}
		
		if(verbose)
			fprintf(stderr, "%s: Launching into the background.  (Daemonizing...)\n", PACKAGE);
			
		/* become a daemon - muahahahaha! */
		if( Daemonize( verbose, doPidFile, pidFileName ) != 0)
		{
			fprintf(stderr, "%s: Unable to become a daemon!\n", PACKAGE);
			exit(1);
		}

	}	/* end if(!resetOnly) */
	
	
	
	/* main loop */
	
	while(!interrupted)
	{
        	if( i < MAXINT )
				i++;
			else
				i = 0;
        	
	
		/* Be sure we still have an active term */
		if(!isatty(ttyDes))
		{
			fprintf(stderr, "%s: %s is no longer an active terminal!\n", PACKAGE, ttyName);
			
			/* Bail out */
			exit(1);
		}
		
		/* Print our line */
		if( -1 == write(ttyDes, cache[i % numChars], strLen) )
		{
			/* If this failed due to a temporarily unavailable code (EAGAIN) then ignore it.  					*/
			/* Mac OSX generates this a lot. It seems to be Terminal.app  as it works OK in XWindows on OS X 	*/
			if(errno == EAGAIN)
			{
				/* Here we send the restore code again. */
				/* The EAGAIN seems to sometimes happen mid-write on OS X... This suucks */
				write(ttyDes, "\0338", 2); /* restore settings */
			}
			else
			{
				snprintf(failMsg, MAX_FAIL_MSG_LEN, "%s: write failed!\n%s: %s\n", PACKAGE, PACKAGE, strerror(errno));
				interrupted = 1; /* we're gonna quit, but got to clenup */
				exitCode = 1;
			}
		}

		if(microTime)
			usleep(time);
		else
			sleep(time);
	
	} /* end main loop */
	
	
	/* clean up pid file */
	if(doPidFile)
	{
		if( 0 != (unlink( pidFileName )) )
		{
			fprintf(stderr, "%s: WARNING: Unable to delete pid file %s.\n%s: WARNING: %s\n", PACKAGE, pidFileName, PACKAGE, strerror(errno));
		}
	}
	
	/* reset term if possible */
	/* This is an attempt to keep the terminal in order when spinner stops. 	*/
	/* Nevertheless, on some systems, we end up with the cursor in the wrong 	*/
	/* place, or in thw wrong mdode, when we are stopped mid-write.				*/
	/* email me if you know how to fix this...									*/
	if( resetOnQuit)
	{	
		if(verbose)
		{
			fprintf(stderr, "%s: Attempting to reset terminal...\n", PACKAGE);
			sleep(1);	
		}
		
		write(ttyDes, "\0338", 2); /* restore settings */
		write(ttyDes, "\033c", 2); /* call reset term */
		
		if(verbose)
		{
			fprintf(stderr, "%s: Terminal reset code written.\n", PACKAGE);
				
		}
	}	
	
	
	/*print fail message if needed - AFTER we reset the term...*/
	fprintf(stderr, "%s", failMsg );
	
	return(exitCode);
}

