/* 
    main.c, Copyright (C) 2001 Joe Laffey     
    Main for Whatmask 

    $Revision: 1.17 $
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
#include "myTypes.h"
#endif


#include "whatnet.h"
#include "usage.h"

#include "main.h"


#if __dest_os == __mac_os
int main(void)
#else
int main(int argc, char** argv)
#endif
{
	
	uint32 val1;
	char	*inStrPtr;
	int		retVal;
	char*	slashPos;
	
#if __dest_os == __mac_os
	char	inStr[256];
	char	line[256];
	
	inStrPtr = inStr; 
	
	printf("%s v%s, Copyright (C) 2001 Joe Laffey <joe@laffeycomputer.com>\n", PACKAGE, VERSION);
	printf("Visit: http://www.laffeycomputer.com/software.html for updates.\n\n");

	printf("%s comes with ABSOLUTELY NO WARRANTY; for details see the COPYING file\nthat accompained this distribution. ", PACKAGE);
	printf("This is free software, and you are welcome\nto redistribute it");
	printf(" under the terms of GNU PUBLIC LICENSE.\n");
	
	usage();
	printf("\n");
	
	/* Loop for ever for Mac OS Classic because there is no command line */
	while(1)
	{
	
	printf("\nEnter the netmask or ip/netmask: ");
	fgets(line, 255, stdin);
	if(sscanf(line,"%s", inStrPtr) != 1)
	{
		usage();
		continue;
	}

#else

	
	if(argc != 2)
		return(usage());
		
	inStrPtr = argv[1];

#endif
	
	
		
	
	if(inStrPtr[0] == '/') /* drop a leading slash */
		inStrPtr += 1;
		
	slashPos = strchr(inStrPtr, '/');
	
		
	
	if(slashPos != NULL)
	{
		/* This is an ip/netmask combo to be processed as net info */
		retVal = whatnet_main(inStrPtr, slashPos);
	}
	else
	{
	
		
		/* try to convert the octets - this also returns 1 if the conversion fails */
		if(octets_to_u_int(&val1, inStrPtr))
		{
			/*assume CIDR */
			if((strchr(inStrPtr, '.') != NULL) || sscanf(inStrPtr, U_INT_32_PRINTF_STRING, &val1) != 1)
			{
				/* not CIDR */
				fprintf(stderr, "%s: \"%s\" is not a valid subnet mask or wildcard bit mask!\n", PACKAGE, inStrPtr);
				retVal = EX_DATAERR;
			}
			else
			{	
				/*try CIDR */
				retVal =  ctonm_main(inStrPtr, val1);
			}
		}
		else 
		{
			/* octet conversion succeeded so we have a netmask */
			retVal =  nmtoc_main(inStrPtr,val1);
		}
	}
#if __dest_os == __mac_os
	} /* end of while(1) loop forever for Mac OS classic */
#endif

	return(retVal);
}

